"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.argon2d = exports.argon2i = void 0;

var _refNapi = _interopRequireDefault(require("ref-napi"));

var _library = require("./library");

var _anyPromise = _interopRequireDefault(require("any-promise"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

const ARGON2_OK = 0;
const ARGON2_VERIFY_MISMATCH = -35;
const defaultOptions = {
  timeCost: 3,
  memoryCost: 1 << 12,
  parallelism: 1,
  hashLength: 32
};

function parseArgs([password, salt, options = {}]) {
  if (typeof password === 'string') {
    password = new Buffer(password);
  }

  options = Object.assign({}, defaultOptions, options);
  return [password, salt, options];
}

function variant(hashRaw, hashEncoded, verify) {
  return {
    hashRaw(...args) {
      const _parseArgs = parseArgs(args),
            _parseArgs2 = _slicedToArray(_parseArgs, 3),
            password = _parseArgs2[0],
            salt = _parseArgs2[1],
            options = _parseArgs2[2];

      const timeCost = options.timeCost,
            memoryCost = options.memoryCost,
            parallelism = options.parallelism,
            hashLength = options.hashLength;
      const hashOutput = new Buffer(hashLength);
      const promise = new _anyPromise.default((resolve, reject) => {
        const resultHandler = (err, res) => {
          if (err) {
            return reject(err);
          }

          if (res !== ARGON2_OK) {
            return reject(new Error((0, _library.argon2ErrorMessage)(res)));
          }

          return resolve(hashOutput);
        };

        hashRaw.async(timeCost, memoryCost, parallelism, password, password.length, salt, salt.length, hashOutput, hashLength, resultHandler);
      });
      return promise;
    },

    hash(...args) {
      const _parseArgs3 = parseArgs(args),
            _parseArgs4 = _slicedToArray(_parseArgs3, 3),
            password = _parseArgs4[0],
            salt = _parseArgs4[1],
            options = _parseArgs4[2];

      const timeCost = options.timeCost,
            memoryCost = options.memoryCost,
            parallelism = options.parallelism,
            hashLength = options.hashLength;
      const encodedSize = (0, _library.argon2Encodedlen)(timeCost, memoryCost, parallelism, salt.length, hashLength);
      const outputBuffer = new Buffer(encodedSize);
      const promise = new _anyPromise.default((resolve, reject) => {
        const resultHandler = (err, res) => {
          if (err) {
            return reject(err);
          }

          if (res !== ARGON2_OK) {
            return reject(new Error((0, _library.argon2ErrorMessage)(res)));
          }

          return resolve(_refNapi.default.readCString(outputBuffer, 0));
        };

        hashEncoded.async(timeCost, memoryCost, parallelism, password, password.length, salt, salt.length, hashLength, outputBuffer, outputBuffer.length, resultHandler);
      });
      return promise;
    },

    verify(encoded, password) {
      const encodedBuffer = _refNapi.default.allocCString(encoded || '');

      let parsedPassword = password;

      if (typeof password === 'string') {
        parsedPassword = new Buffer(password);
      }

      const promise = new _anyPromise.default((resolve, reject) => {
        const resultHandler = (err, res) => {
          if (err) {
            return reject(err);
          }

          if (res === ARGON2_OK) {
            return resolve(true);
          } else if (res === ARGON2_VERIFY_MISMATCH) {
            return resolve(false);
          } else {
            return reject(new Error((0, _library.argon2ErrorMessage)(res)));
          }
        };

        verify.async(encodedBuffer, parsedPassword, parsedPassword.length, resultHandler);
      });
      return promise;
    }

  };
}

const argon2i = variant(_library.argon2iHashRaw, _library.argon2iHashEncoded, _library.argon2iVerify);
exports.argon2i = argon2i;
const argon2d = variant(_library.argon2dHashRaw, _library.argon2dHashEncoded, _library.argon2dVerify);
exports.argon2d = argon2d;