"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.argon2dVerify = exports.argon2iVerify = exports.argon2dHashRaw = exports.argon2dHashEncoded = exports.argon2iHashRaw = exports.argon2iHashEncoded = exports.argon2Encodedlen = exports.argon2ErrorMessage = void 0;

var _ffiNapi = _interopRequireDefault(require("ffi-napi"));

var _refNapi = _interopRequireDefault(require("ref-napi"));

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const RELEASE_DIR = _path.default.join(__dirname, '..', 'build', 'Release');

const LIBRARY_SEARCH_PATHS = [_path.default.join(RELEASE_DIR, 'argon2'), _path.default.join(RELEASE_DIR, 'obj.target', 'argon2')];

function getLibraryPath() {
  const expectedExtensions = ['so', 'dylib', 'dll'];

  for (var _i = 0; _i < LIBRARY_SEARCH_PATHS.length; _i++) {
    const searchPath = LIBRARY_SEARCH_PATHS[_i];

    for (var _i2 = 0; _i2 < expectedExtensions.length; _i2++) {
      const extension = expectedExtensions[_i2];
      const libraryPath = `${searchPath}.${extension}`;

      if (_fs.default.existsSync(libraryPath) && _fs.default.lstatSync(libraryPath).isFile()) {
        return libraryPath;
      }
    }
  }

  throw new Error('Could not find argon2 library');
}

const lib = new _ffiNapi.default.Library(getLibraryPath(), {
  argon2i_hash_encoded: ['int', ['uint32', 'uint32', 'uint32', // t_cost, m_cost, p
  _refNapi.default.refType('void'), 'size_t', // password
  _refNapi.default.refType('void'), 'size_t', // salt
  'size_t', // hash length
  _refNapi.default.refType('char'), 'size_t']],
  // encoded hash
  argon2i_hash_raw: ['int', ['uint32', 'uint32', 'uint32', // t_cost, m_cost, p
  _refNapi.default.refType('void'), 'size_t', // password
  _refNapi.default.refType('void'), 'size_t', // salt
  _refNapi.default.refType('void'), 'size_t']],
  // hash output
  argon2d_hash_encoded: ['int', ['uint32', 'uint32', 'uint32', // t_cost, m_cost, p
  _refNapi.default.refType('void'), 'size_t', // password
  _refNapi.default.refType('void'), 'size_t', // salt
  'size_t', // hash length
  _refNapi.default.refType('char'), 'size_t']],
  // encoded hash
  argon2d_hash_raw: ['int', ['uint32', 'uint32', 'uint32', // t_cost, m_cost, p
  _refNapi.default.refType('void'), 'size_t', // password
  _refNapi.default.refType('void'), 'size_t', // salt
  _refNapi.default.refType('void'), 'size_t']],
  // hash output
  argon2i_verify: ['int', [_refNapi.default.refType('char'), // encoded
  _refNapi.default.refType('void'), // password
  'size_t']],
  // password length
  argon2d_verify: ['int', [_refNapi.default.refType('char'), // encoded
  _refNapi.default.refType('void'), // password
  'size_t']],
  // password length
  argon2_encodedlen: ['size_t', ['uint32', 'uint32', 'uint32', // t_cost, m_cost, p
  'uint32', 'uint32']],
  // salt length, hash length
  argon2_error_message: ['string', ['int']] // error_code

});
const argon2ErrorMessage = lib.argon2_error_message,
      argon2Encodedlen = lib.argon2_encodedlen,
      argon2iHashEncoded = lib.argon2i_hash_encoded,
      argon2iHashRaw = lib.argon2i_hash_raw,
      argon2dHashEncoded = lib.argon2d_hash_encoded,
      argon2dHashRaw = lib.argon2d_hash_raw,
      argon2iVerify = lib.argon2i_verify,
      argon2dVerify = lib.argon2d_verify;
exports.argon2dVerify = argon2dVerify;
exports.argon2iVerify = argon2iVerify;
exports.argon2dHashRaw = argon2dHashRaw;
exports.argon2dHashEncoded = argon2dHashEncoded;
exports.argon2iHashRaw = argon2iHashRaw;
exports.argon2iHashEncoded = argon2iHashEncoded;
exports.argon2Encodedlen = argon2Encodedlen;
exports.argon2ErrorMessage = argon2ErrorMessage;